#!/usr/bin/python3.6.8+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-9-22
@desc: 权限验证的装饰器模块
"""
from yzcore.request import request
from yzcore.exceptions import NoPermission


PERMISSION_URL = 'http://localhost:9001/api/permission/'


class CheckPermission:
    def __init__(self):
        pass

    def __call__(self, func):
        def _call(permission, *args, **kw):
            print('===>decorator of class is runing')
            user_id = kw.get('user_id')
            space_id = kw.get('space_id')
            if self._validate(permission, user_id, space_id):
                return func(*args, **kw)
            else:
                raise NoPermission()
        return _call

    def _validate(self, permission, space_id, user_id):
        """"""
        # 请求参数
        data = dict(
            object_id=space_id,
            object_type=1,
            space_id=space_id,
            user_id=user_id
        )
        # 发送请求验证
        url = PERMISSION_URL
        response, status = request('get', url, json=data)
        # 判断结果
        if status == 200:
            _permission = response.get('permission')
            if permission >= _permission:
                return True
        return False


def singleton(cls, *args, **kwargs):
    """构造一个单例的装饰器"""
    instance = {}

    def __singleton(*args, **kwargs):
        if cls not in instance:
            instance[cls] = cls(*args, **kwargs)
        return instance[cls]

    return __singleton


if __name__ == '__main__':
    pass
# class Bar:
#     @Foo()
#     def bar(self):
#         print('==>exec(bar)')



"""
nacos sdk 支持版本为Python 2.7 Python 3.6 Python 3.7
"""
import nacos
import logging

class BaseNacosClient:
    def __init__(self,nacos_url:str,username:str,password:str,service_name:str,nacos_ip:str=None,nacos_port:int=None,namespace:str=None,**kwargs):
        """
        param: url (url of nacos)
        param: namespace
        param: username (username of nacos)
        param: password (password of nacos)
        param: service_name
        param: ip (ip of nacos)
        param: port (port of nacos)
        """
        self.url = nacos_url
        self.namespace = namespace
        self.username = username
        self.password = password
        self.service_name = service_name
        self.nacos_ip = nacos_ip
        self.nacos_port = nacos_port
        self.client = nacos.NacosClient(nacos_url,namespace=namespace,username=username,password=password)

    async def send_heartbeat(self):
        """向nacos发送心跳"""
        try:
            self.client.send_heartbeat(
                self.service_name,
                ip=self.nacos_ip,
                port=self.nacos_port
                )
        except Exception:
            # 如果没有收到心跳返回信息则重新注册
            self.client.add_naming_instance(
                service_name=self.service_name,
                ip=self.nacos_ip,
                port=self.nacos_port
            )


    def register_nacos(self):
        """微服务注册nacos"""
        try:
            self.client.add_naming_instance(
                service_name=self.service_name,
                ip=self.nacos_ip,
                port=self.nacos_port
            )
        except Exception as ex:
            logging.error("Failed to register to nacos : %s", ex)



#!/usr/bin/python3.7+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-9-
@desc: ...
"""
"""
Django's standard crypto functions and utilities.
"""
import hashlib
import hmac
import random
import time

from .encoding import force_bytes
# from yzcore.core.default_settings import get_settings
#
# settings = get_settings()

# Use the system PRNG if possible
try:
    random = random.SystemRandom()
    using_sysrandom = True
except NotImplementedError:
    import warnings
    warnings.warn('A secure pseudo-random number generator is not available '
                  'on your system. Falling back to Mersenne Twister.')
    using_sysrandom = False



def get_random_string(length=12,
                      allowed_chars='abcdefghijklmnopqrstuvwxyz'
                                    'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'):
    """
    Return a securely generated random string.

    The default length of 12 with the a-z, A-Z, 0-9 character set returns
    a 71-bit value. log_2((26+26+10)^12) =~ 71 bits
    """
    # if not using_sysrandom:
    #     # This is ugly, and a hack, but it makes things better than
    #     # the alternative of predictability. This re-seeds the PRNG
    #     # using a value that is hard for an attacker to predict, every
    #     # time a random string is required. This may change the
    #     # properties of the chosen random sequence slightly, but this
    #     # is better than absolute predictability.
    #     random.seed(
    #         hashlib.sha256(
    #             ('%s%s%s' % (random.getstate(), time.time(), settings.SECRET_KEY)).encode()
    #         ).digest()
    #     )
    return ''.join(random.choice(allowed_chars) for i in range(length))


def constant_time_compare(val1, val2):
    """Return True if the two strings are equal, False otherwise."""
    return hmac.compare_digest(force_bytes(val1), force_bytes(val2))


def pbkdf2(password, salt, iterations, dklen=0, digest=None):
    """Return the hash of password using pbkdf2."""
    if digest is None:
        digest = hashlib.sha256
    dklen = dklen or None
    password = force_bytes(password)
    salt = force_bytes(salt)
    return hashlib.pbkdf2_hmac(digest().name, password, salt, iterations, dklen)



from yzcore.extensions.storage import StorageManage, StorageRequestError
from abc import ABCMeta, abstractmethod


__all__ = [
    'StorageRequestError',
    'StorageController',
    'StorageManage',
]


class StorageController(metaclass=ABCMeta):
    """
    对象存储控制器
    组织自定义对象存储
    >>> storage_ctrl = await StorageController.init(organiz_id='organiz_id')
    >>> storage_ctrl.organiz_storage_conf  # 组织自定义对象存储配置，未配置则为空字典
    >>> storage_ctrl.storage_conf  # 组织自定义对象存储配置或全局配置
    >>> storage_ctrl.public_storage_manage  # 非加密存储控制器
    >>> storage_ctrl.private_storage_manage  # 加密存储控制器
    全局对象存储
    >>> global_storage_manage = await StorageController.init()
    >>> global_storage_manage.public_storage_manage  # 全局非加密存储控制器
    >>> global_storage_manage.private_storage_manage  # 全局加密存储控制器
    """

    def __init__(self, organiz_id):
        """
        不可直接使用，请在 StorageManage.init 初始化
        """
        self.organiz_id = organiz_id
        self.organiz_storage_conf = dict()
        self.storage_conf = dict()
        self.storage_mode = ''

    @classmethod
    async def init(cls, organiz_id: str = ''):
        """
        organiz_id为空的情况下返回全局的对象存储
        因在 __init__ 方法中不能调用异步方法，需要在这里初始化"""
        storage_ctrl = cls(organiz_id)
        # 获取组织自定义对象存储配置
        if organiz_id:
            organiz_storage_conf = await storage_ctrl._get_organiz_storage_conf()
            if organiz_storage_conf:
                # 覆盖回全局对象存储配置，避免丢失全局配置
                _storage_conf = cls.global_storage_conf().copy()
                _storage_conf.update(organiz_storage_conf)
                storage_ctrl.organiz_storage_conf = _storage_conf
        storage_ctrl.storage_conf = storage_ctrl.organiz_storage_conf or cls.global_storage_conf()
        storage_ctrl.storage_mode = storage_ctrl.storage_conf['mode']
        return storage_ctrl

    @classmethod
    def sync_init(cls):
        """同步方式初始化，只适用于全局存储的初始化"""
        storage_ctrl = cls('')
        storage_ctrl.storage_conf = cls.global_storage_conf().copy()
        storage_ctrl.storage_mode = storage_ctrl.storage_conf['mode']
        return storage_ctrl

    @abstractmethod
    async def _get_organiz_storage_conf(self):
        """
        获取组织的自定义对象存储配置
        :return: dict(**organiz_storage_conf) or None
        """

    @classmethod
    @abstractmethod
    def global_storage_conf(cls):
        """返回全局对象存储配置"""

    @property
    def public_storage_manage(self):
        """非加密存储桶控制器"""
        return self._init_public_storage_manage(self.storage_conf)

    @property
    def private_storage_manage(self):
        """加密存储桶控制器"""
        return self._init_private_storage_manage(self.storage_conf)

    @classmethod
    async def check_organiz_conf(cls, organiz_conf: dict):
        """检查自定义对象存储配置是否有效"""
        public_storage = cls._init_public_storage_manage(organiz_conf)
        private_storage = cls._init_private_storage_manage(organiz_conf)
        public_storage.check()
        private_storage.check()

    @classmethod
    def _init_public_storage_manage(cls, storage_conf: dict):
        """
        初始化非加密存储桶
        :param storage_conf:
            字段:
            mode: str
            access_key_id: str
            access_key_secret: str
            endpoint: str
            public_bucket_name: str             # 非加密桶使用
            private_bucket_name: Optional[str]  # 加密桶使用
            image_domain: Optional[str]         # 非加密桶使用，可选
            asset_domain: Optional[str]         # 非加密桶使用，可选
            private_domain: Optional[str]       # 加密桶使用，可选
            private_cname: Optional[str]        # 加密桶使用，可选

        :return: _StorageManage实例，即 ObsManager 或 OssManager
        """
        return StorageManage(
            mode=storage_conf['mode'],
            access_key_id=storage_conf['access_key_id'],
            access_key_secret=storage_conf['access_key_secret'],
            endpoint=storage_conf['endpoint'],
            bucket_name=storage_conf['public_bucket_name'],  # 注意区分加密/非加密存储桶
            image_domain=storage_conf['image_domain'],
            asset_domain=storage_conf['asset_domain'],
            scheme=storage_conf.get('scheme', 'https'),
            cache_path=cls.global_storage_conf()['cache_path'],  # 来自全局配置
            policy_expire_time=cls.global_storage_conf()['policy_expire_time'],  # 来自全局配置
            private_expire_time=cls.global_storage_conf()['private_expire_time'],  # 来自全局配置
        )

    @classmethod
    def _init_private_storage_manage(cls, storage_conf: dict):
        """
        初始化加密存储桶
        :param storage_conf:
            字段:
            mode: str
            access_key_id: str
            access_key_secret: str
            endpoint: str
            public_bucket_name: Optional[str]   # 非加密桶使用
            private_bucket_name: str            # 加密桶使用
            image_domain: Optional[str]         # 非加密桶使用，可选
            asset_domain: Optional[str]         # 非加密桶使用，可选
            private_domain: Optional[str]       # 加密桶使用，可选
            private_cname: Optional[str]        # 加密桶使用，可选

        :return: _StorageManage实例，即 ObsManager 或 OssManager
        """
        return StorageManage(
            mode=storage_conf['mode'],
            access_key_id=storage_conf['access_key_id'],
            access_key_secret=storage_conf['access_key_secret'],
            endpoint=storage_conf['endpoint'],
            bucket_name=storage_conf['private_bucket_name'],  # 注意区分加密/非加密存储桶
            image_domain=storage_conf['private_domain'],
            asset_domain=storage_conf['private_domain'],
            cname=storage_conf['private_cname'],
            scheme=storage_conf.get('scheme', 'https'),
            cache_path=cls.global_storage_conf()['cache_path'],  # 来自全局配置
            policy_expire_time=cls.global_storage_conf()['policy_expire_time'],  # 来自全局配置
            private_expire_time=cls.global_storage_conf()['private_expire_time'],  # 来自全局配置
        )



#!/usr/bin/python3.6.8+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-12-2
@desc: ...
"""


class JobStatus(object):
    PENDING = 0  # 任务等待执行

    STARTED = 100  # 任务执行开始
    PROCESS = 110
    POLLING = 120
    CALLBACK = 130

    SUCCESS = 200   # 任务执行成功
    RETRY = 300     # 任务重试
    FAILURE = 400   # 任务执行失败
    REVOKED = 500   # 任务撤销




#!/usr/bin/python3.7+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-9-1
@desc: ...
"""
import os


def mkdir_if_not_exist(path: str):
    """不存在就创建目录"""
    if not os.path.exists(path):
        os.makedirs(path)


#!/usr/bin/python3.6.8+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-10-20
@desc: 对阿里云oss的封装，依赖oss2
"""

import os
import json
import base64
import hmac
import datetime
import hashlib
from urllib import parse
from yzcore.extensions.storage.base import StorageManagerBase, StorageRequestError, IMAGE_FORMAT_SET
from yzcore.extensions.storage.oss.const import *

try:
    import oss2
    from oss2 import CaseInsensitiveDict
except:
    oss2 = None


class OssManager(StorageManagerBase):

    def __init__(self, *args, **kwargs):
        super(OssManager, self).__init__(*args, **kwargs)
        self.__init()

    def __init(self, bucket_name=None):
        """初始化对象"""

        if oss2 is None:
            raise ImportError("'oss2' must be installed to use OssManager")
        if not any((self.endpoint, self.cname)):
            raise AttributeError(
                "One of 'endpoint' and 'cname' must not be None.")

        self.auth = oss2.Auth(self.access_key_id, self.access_key_secret)

        # 首先，如果cname存在，则使用自定义域名初始化
        if self.cname:
            self.endpoint = self.cname
            self.is_cname = True
        # 其次，优先内网endpoint
        elif self.internal_endpoint:
            self.endpoint = self.internal_endpoint

        self.bucket_name = bucket_name if bucket_name else self.bucket_name
        self.bucket = oss2.Bucket(
            self.auth, self.endpoint, self.bucket_name, is_cname=self.is_cname
        )

        if self.cache_path:
            try:
                os.makedirs(self.cache_path)
            except OSError:
                pass

    def reload_oss(self, **kwargs):
        """重新加载oss配置"""
        self.access_key_id = kwargs.get("access_key_id")
        self.access_key_secret = kwargs.get("access_key_secret")
        self.bucket_name = kwargs.get("bucket_name")
        self.endpoint = kwargs.get("endpoint")
        self.__init()

    def create_bucket(self, bucket_name=None,
                      acl_type='private',
                      storage_type='standard',
                      redundancy_type='zrs'):
        """创建bucket，并且作为当前操作bucket"""
        permission = ACL_TYPE.get(acl_type)
        config = oss2.models.BucketCreateConfig(
            storage_class=STORAGE_CLS.get(storage_type),
            data_redundancy_type=REDUNDANCY_TYPE.get(redundancy_type)
        )
        result = self.bucket.create_bucket(permission, input=config)
        self.__init(bucket_name=bucket_name)
        return result

    def get_bucket_cors(self):
        """获取存储桶的CORS配置"""
        cors_dict = {
            'allowed_origins': [],
            'allowed_methods': [],
            'allowed_headers': [],
        }
        try:
            cors = self.bucket.get_bucket_cors()
            for rule in cors.rules:
                cors_dict['allowed_origins'] = rule.allowed_origins
                cors_dict['allowed_headers'] = rule.allowed_headers
                cors_dict['allowed_methods'] = rule.allowed_methods
        except oss2.exceptions.NoSuchCors:
            pass
        return cors_dict

    def iter_buckets(self, prefix='', marker='', max_keys=100, max_retries=None):
        """
        :param prefix: 只列举匹配该前缀的Bucket
        :param marker: 分页符。只列举Bucket名字典序在此之后的Bucket
        :param max_keys: 每次调用 `list_buckets` 时的max_keys参数。注意迭代器返回的数目可能会大于该值。
        :param max_retries:
        :return:
        """
        if not hasattr(self, 'service'):
            self.service = oss2.Service(self.auth, self.endpoint)

        return oss2.BucketIterator(
            self.service, prefix=prefix, marker=marker,
            max_keys=max_keys, max_retries=max_retries)

    def list_buckets(self, prefix='', marker='', max_keys=100, params=None):
        """根据前缀罗列用户的Bucket。

        :param str prefix: 只罗列Bucket名为该前缀的Bucket，空串表示罗列所有的Bucket
        :param str marker: 分页标志。首次调用传空串，后续使用返回值中的next_marker
        :param int max_keys: 每次调用最多返回的Bucket数目
        :param dict params: list操作参数，传入'tag-key','tag-value'对结果进行过滤

        :return: 罗列的结果
        :rtype: oss2.models.ListBucketsResult
        """
        if not hasattr(self, 'service'):
            self.service = oss2.Service(self.auth, self.endpoint)
        return self.service.list_buckets(
            prefix=prefix, marker=marker, max_keys=max_keys, params=params)

    def is_exist_bucket(self, bucket_name=None):
        """判断存储空间是否存在"""
        try:
            self.bucket.get_bucket_info()
        except oss2.exceptions.NoSuchBucket:
            return False
        except:
            raise
        return True

    def delete_bucket(self, bucket_name=None):
        """删除bucket"""
        try:
            resp = self.bucket.delete_bucket()
            if resp.status < 300:
                return True
            elif resp.status == 404:
                return False
        except:
            import traceback
            print(traceback.format_exc())

    def encrypt_bucket(self):
        """加密bucket"""
        # 创建Bucket加密配置，以AES256加密为例。
        rule = oss2.models.ServerSideEncryptionRule()
        rule.sse_algorithm = oss2.SERVER_SIDE_ENCRYPTION_AES256
        # 设置KMS密钥ID，加密方式为KMS可设置此项。
        # 如需使用指定的密钥加密，需输入指定的CMK ID；
        # 若使用OSS托管的CMK进行加密，此项为空。使用AES256进行加密时，此项必须为空。
        rule.kms_master_keyid = ""

        # 设置Bucket加密。
        result = self.bucket.put_bucket_encryption(rule)
        # 查看HTTP返回码。
        print('http response code:', result.status)
        return result

    def delete_encrypt_bucket(self):
        # 删除Bucket加密配置。
        result = self.bucket.delete_bucket_encryption()
        # 查看HTTP返回码。
        print('http status:', result.status)
        return result

    def get_sign_url(self, key, expire=0):
        url = self.bucket.sign_url("GET", key, expire or self.private_expire_time)
        return '//' + url.split('//', 1)[-1]

    def post_sign_url(self, key):
        pass

    def put_sign_url(self, key):
        return self.bucket.sign_url("PUT", key, self.policy_expire_time)

    def delete_cache_file(self, filename):
        """删除文件缓存"""
        filepath = os.path.abspath(os.path.join(self.cache_path, filename))
        assert os.path.isfile(filepath), '非文件或文件不存在'
        os.remove(filepath)

    def search_cache_file(self, filename):
        """文件缓存搜索"""
        # 拼接绝对路径
        filepath = os.path.abspath(os.path.join(self.cache_path, filename))
        if os.path.isfile(filepath):
            return filepath
        else:
            return None

    def iter_objects(self, prefix='', marker='', delimiter='', max_keys=100):
        """
        遍历bucket下的文件
        :param prefix: key前缀
        :param marker:
        :param delimiter:
        :param max_keys:
        :return: dict
        """
        _result = []
        for obj in oss2.ObjectIterator(self.bucket, prefix=prefix, marker=marker, delimiter=delimiter, max_keys=max_keys):
            _result.append({
                'key': obj.key,
                'url': self.get_file_url(key=obj.key),
                'size': obj.size,
            })
        return _result

    def download_stream(self, key, process=None):
        return self.bucket.get_object(key, process=process)

    def download_file(self, key, local_name, process=None):
        self.bucket.get_object_to_file(key, local_name, process=process)

    def upload(self, filepath, key: str, num_threads=2, multipart_threshold=None):
        """
        上传oss文件
        :param filepath:
        :param key:
        :param num_threads:
        :param multipart_threshold:
        """
        headers = CaseInsensitiveDict({'Content-Type': self.parse_content_type(key)})

        if isinstance(filepath, str):
            result = oss2.resumable_upload(
                self.bucket, key, filepath,
                headers=headers,
                num_threads=num_threads,
                multipart_threshold=multipart_threshold,
            )
        else:
            result = self.bucket.put_object(key, filepath, headers=headers)
        if result.status != 200:
            raise StorageRequestError(f'oss upload error: {result.resp}')
        # 返回下载链接
        return self.get_file_url(key)

    def get_policy(
            self,
            filepath: str,
            callback_url: str,
            callback_data: dict = None,
            callback_content_type: str = "application/json"):
        """
        授权给第三方上传

        :param filepath:
        :param callback_url:
        :param callback_data: 需要回传的参数
        :param callback_content_type: 回调时的Content-Type
               "application/json"
               "application/x-www-form-urlencoded"

        :return:
        """
        params = parse.urlencode(
            dict(data=json.dumps(callback_data)))
        policy_encode = self._get_policy_encode(filepath)
        sign = self.get_signature(policy_encode)

        callback_dict = dict()
        callback_dict["callbackUrl"] = callback_url
        callback_dict["callbackBody"] = (
            "filepath=${object}&size=${size}&mime_type=${mimeType}"
            "&etag=${etag}"
            "&img_height=${imageInfo.height}&img_width=${imageInfo.width}"
            "&img_format=${imageInfo.format}&" + params
        )
        callback_dict["callbackBodyType"] = callback_content_type

        callback_param = json.dumps(callback_dict).strip().encode()
        base64_callback_body = base64.b64encode(callback_param)

        return dict(
            mode='oss',
            OSSAccessKeyId=self.access_key_id,
            host=f'{self.scheme}:{self.host}',
            policy=policy_encode.decode(),
            signature=sign,
            dir=filepath,
            callback=base64_callback_body.decode(),
        )

    def _get_policy_encode(self, filepath):
        expire_time = datetime.datetime.now() + datetime.timedelta(
            seconds=self.policy_expire_time
        )
        policy_dict = dict(
            expiration=expire_time.isoformat() + "Z",
            conditions=[
                ["starts-with", "$key", filepath],                    # 指定值开始
                # ["eq", "$success_action_redirect", "public-read"],  # 精确匹配
                # ["content-length-range", 1, 1024*1024*1024]         # 对象大小限制
            ],
        )
        policy = json.dumps(policy_dict).strip().encode()
        return base64.b64encode(policy)

    def get_signature(self, policy_encode):
        """
        获取签名

        :param policy_encode:
        :return:
        """
        h = hmac.new(
            self.access_key_secret.encode("utf-8"), policy_encode, hashlib.sha1
        )
        sign_result = base64.encodebytes(h.digest()).strip()
        return sign_result.decode()

    def update_file_headers(self, key, headers):
        self.bucket.update_object_meta(key, headers)
        return True

    def file_exists(self, key):
        """检查文件是否存在"""
        return self.bucket.object_exists(key)

    def get_object_meta(self, key: str):
        """获取文件基本元信息，包括该Object的ETag、Size（文件大小）、LastModified，Content-Type，并不返回其内容"""
        # meta = self.bucket.get_object_meta(key)  # get_object_meta获取到的信息有限
        meta = self.bucket.head_object(key)
        return {
            'etag': meta.etag.lower(),
            'size': meta.content_length,
            'last_modified': meta.headers['Last-Modified'],
            'content_type': meta.headers['Content-Type']
        }

    @property
    def host(self):
        if self.cname:
            return u'//{}'.format(self.cname)
        else:
            return u'//{}.{}'.format(self.bucket_name, self.endpoint.replace('-internal', ''))

    def get_file_url(self, key):
        if not any((self.image_domain, self.asset_domain)):
            if self.is_cname:
                resource_url = u"//{}/{}".format(self.cname, key)
            else:
                resource_url = u"//{}.{}/{}".format(self.bucket_name, self.endpoint.replace('-internal', ''), key)
        elif key.split('.')[-1].lower() in IMAGE_FORMAT_SET:
            resource_url = u"//{domain}/{key}".format(domain=self.image_domain, key=key)
        else:
            resource_url = u"//{domain}/{key}".format(domain=self.asset_domain, key=key)
        return resource_url



# -*- coding: utf-8 -*-
import os
from oss2 import SizedFileAdapter, determine_part_size
from oss2.models import PartInfo
import oss2

# 阿里云账号AccessKey拥有所有API的访问权限，风险很高。强烈建议您创建并使用RAM用户进行API访问或日常运维，请登录RAM控制台创建RAM用户。
auth = oss2.Auth('yourAccessKeyId', 'yourAccessKeySecret')
# Endpoint以华东1（杭州）为例，其他Region请按实际情况填写。
# 填写Bucket名称，例如examplebucket。
bucket = oss2.Bucket(auth, 'https://oss-cn-hangzhou.aliyuncs.com', 'examplebucket')
# 填写不能包含Bucket名称在内的Object完整路径，例如exampledir/exampleobject.txt。
key = 'exampledir/exampleobject.txt'
# 填写本地文件的完整路径，例如D:\\localpath\\examplefile.txt。
filename = 'D:\\localpath\\examplefile.txt'

total_size = os.path.getsize(filename)
# determine_part_size方法用于确定分片大小。
part_size = determine_part_size(total_size, preferred_size=100 * 1024)

# 初始化分片。
# 如需在初始化分片时设置文件存储类型，请在init_multipart_upload中设置相关Headers，参考如下。
# headers = dict()
# 指定该Object的网页缓存行为。
# headers['Cache-Control'] = 'no-cache'
# 指定该Object被下载时的名称。
# headers['Content-Disposition'] = 'oss_MultipartUpload.txt'
# 指定该Object的内容编码格式。
# headers['Content-Encoding'] = 'utf-8'
# 指定过期时间，单位为毫秒。
# headers['Expires'] = '1000'
# 指定初始化分片上传时是否覆盖同名Object。此处设置为true，表示禁止覆盖同名Object。
# headers['x-oss-forbid-overwrite'] = 'true'
# 指定上传该Object的每个Part时使用的服务器端加密方式。
# headers[OSS_SERVER_SIDE_ENCRYPTION] = SERVER_SIDE_ENCRYPTION_KMS
# 指定Object的加密算法。如果未指定此选项，表明Object使用AES256加密算法。
# headers[OSS_SERVER_SIDE_DATA_ENCRYPTION] = SERVER_SIDE_ENCRYPTION_KMS
# 表示KMS托管的用户主密钥。
# headers[OSS_SERVER_SIDE_ENCRYPTION_KEY_ID] = '9468da86-3509-4f8d-a61e-6eab1eac****'
# 指定Object的存储类型。
# headers['x-oss-storage-class'] = oss2.BUCKET_STORAGE_CLASS_STANDARD
# 指定Object的对象标签，可同时设置多个标签。
# headers[OSS_OBJECT_TAGGING] = 'k1=v1&k2=v2&k3=v3'
# upload_id = bucket.init_multipart_upload(key, headers=headers).upload_id
upload_id = bucket.init_multipart_upload(key).upload_id
parts = []

# 逐个上传分片。
with open(filename, 'rb') as fileobj:
    part_number = 1
    offset = 0
    while offset < total_size:
        num_to_upload = min(part_size, total_size - offset)
        # 调用SizedFileAdapter(fileobj, size)方法会生成一个新的文件对象，重新计算起始追加位置。
        result = bucket.upload_part(key, upload_id, part_number,
                                    SizedFileAdapter(fileobj, num_to_upload))
        parts.append(PartInfo(part_number, result.etag))

        offset += num_to_upload
        part_number += 1

# 完成分片上传。
# 如需在完成分片上传时设置相关Headers，请参考如下示例代码。
headers = dict()
# 设置文件访问权限ACL。此处设置为OBJECT_ACL_PRIVATE，表示私有权限。
# headers["x-oss-object-acl"] = oss2.OBJECT_ACL_PRIVATE
bucket.complete_multipart_upload(key, upload_id, parts, headers=headers)
# bucket.complete_multipart_upload(key, upload_id, parts)

# 验证分片上传。
with open(filename, 'rb') as fileobj:
    assert bucket.get_object(key).read() == fileobj.read()



import functools
from io import BytesIO
from yzcore.utils.crypto import get_random_string


def create_temp_file(text_length=16):
    """创建一个包含随机字符串的内存文件"""
    file = BytesIO(get_random_string(text_length).encode())
    file.seek(0)
    return file



import os
import shutil
from typing import Union
from io import BufferedReader
from abc import ABCMeta, abstractmethod
from yzcore.extensions.storage.utils import create_temp_file
from yzcore.extensions.storage.const import IMAGE_FORMAT_SET, CONTENT_TYPE
from yzcore.logger import get_logger
from urllib.request import urlopen
from urllib.error import URLError
from urllib.parse import urlparse, unquote
from ssl import SSLCertVerificationError

logger = get_logger(__name__)


class StorageRequestError(Exception):
    """请求外部对象存储服务时遇到的错误或异常"""


class StorageManagerBase(metaclass=ABCMeta):
    def __init__(
            self,
            access_key_id,
            access_key_secret,
            bucket_name,
            endpoint=None,
            internal_endpoint=None,
            cname=None,
            cache_path='.',
            expire_time=30,
            mode=None,
            **kwargs
    ):
        self.access_key_id = access_key_id
        self.access_key_secret = access_key_secret
        self.bucket_name = bucket_name
        self.endpoint = endpoint
        self.internal_endpoint = internal_endpoint

        self.cache_path = cache_path
        self.scheme = kwargs.get("scheme", "https")
        self.image_domain = kwargs.get("image_domain")
        self.asset_domain = kwargs.get("asset_domain")
        self.policy_expire_time = kwargs.get("policy_expire_time", expire_time)
        self.private_expire_time = kwargs.get("private_expire_time", expire_time)

        self.cname = cname
        self.mode = mode
        self.bucket = None
        self.is_cname = False

    @abstractmethod
    def create_bucket(self, bucket_name):
        """创建bucket"""

    @abstractmethod
    def get_bucket_cors(self):
        """
        获取CORS配置
        :return: {
            'allowed_origins': [],
            'allowed_methods': [],
            'allowed_headers': [],
        }
        """

    @abstractmethod
    def list_buckets(self):
        """查询bucket列表"""

    @abstractmethod
    def is_exist_bucket(self, bucket_name=None):
        """判断bucket是否存在"""

    @abstractmethod
    def delete_bucket(self, bucket_name=None):
        """删除bucket"""

    @abstractmethod
    def get_sign_url(self, key, expire=0):
        """生成下载对象的带授权信息的URL"""

    @abstractmethod
    def post_sign_url(self, key):
        """生成POST上传对象的授权信息"""

    @abstractmethod
    def put_sign_url(self, key):
        """生成PUT上传对象的带授权信息的URL"""

    @abstractmethod
    def iter_objects(self, prefix='', marker=None, delimiter=None, max_keys=100):
        """
        遍历存储桶内的文件
        目前返回的字段：
            [{
                'key': '',
                'url: '',
                'size': '',
            }]
        """

    @abstractmethod
    def get_object_meta(self, key: str):
        """获取文件基本元信息，包括该Object的ETag、Size（文件大小）、LastModified，Content-Type，并不返回其内容"""

    @abstractmethod
    def update_file_headers(self, key, headers: dict):
        """更改Object的元数据信息，包括Content-Type这类标准的HTTP头部"""

    @abstractmethod
    def file_exists(self, key):
        """检查文件是否存在"""

    def download(self, key, local_name=None, path=None, is_stream=False, **kwargs):
        """
        下载文件
        :param key:
        :param local_name: 下载的文件在本地的路径
        :param path: 文件下载路径
        :param is_stream:
            is_stream = True:
                >>> result = self.download('readme.txt', is_stream=True)
                >>> print(result.read())
                'hello world'
            is_stream = False:
                >>> result = self.download('readme.txt', '/tmp/cache/readme.txt')
                >>> print(result)
                '/tmp/cache/readme.txt'
        :return: 文件对象或文件下载后的本地路径
        """
        if is_stream:
            return self.download_stream(key, **kwargs)
        else:
            if not local_name:
                if path:
                    local_name = os.path.abspath(os.path.join(self.cache_path, path, self.get_filename(key)))
                else:
                    local_name = os.path.abspath(os.path.join(self.cache_path, key))
            self.make_dir(os.path.dirname(local_name))
            self.download_file(key, local_name)
            return local_name

    @abstractmethod
    def download_stream(self, key, **kwargs):
        """下载文件流"""
        pass

    @abstractmethod
    def download_file(self, key, local_name, **kwargs):
        """下载文件"""
        pass

    @abstractmethod
    def upload(self, filepath: Union[str, BufferedReader], key: str):
        """"""

    @abstractmethod
    def get_policy(
            self,
            filepath: str,
            callback_url: str,
            callback_data: dict = None,
            callback_content_type: str = "application/json"
    ):
        """
        授权给第三方上传
        :param filepath:
        :param callback_url: 对象存储的回调地址
        :param callback_data: 需要回传的参数
        :param callback_content_type: 回调时的Content-Type
               "application/json"
               "application/x-www-form-urlencoded"
        :return:
        """

    @property
    def host(self):
        if self.cname:
            return u'//{}'.format(self.cname)
        else:
            return u'//{}.{}'.format(self.bucket_name, self.endpoint)

    def get_file_url(self, key):
        if not any((self.image_domain, self.asset_domain)):
            if self.is_cname:
                resource_url = u"//{}/{}".format(self.cname, key)
            else:
                resource_url = u"//{}.{}/{}".format(self.bucket_name, self.endpoint, key)
        elif key.split('.')[-1].lower() in IMAGE_FORMAT_SET:
            resource_url = u"//{domain}/{key}".format(domain=self.image_domain, key=key)
        else:
            resource_url = u"//{domain}/{key}".format(domain=self.asset_domain, key=key)
        return resource_url

    def delete_cache_file(self, filename):
        """删除文件缓存"""
        filepath = os.path.abspath(os.path.join(self.cache_path, filename))
        assert os.path.isfile(filepath), '非文件或文件不存在'
        os.remove(filepath)

    def search_cache_file(self, filename):
        """文件缓存搜索"""
        # 拼接绝对路径
        filepath = os.path.abspath(os.path.join(self.cache_path, filename))
        if os.path.isfile(filepath):
            return filepath
        else:
            return None

    @classmethod
    def make_dir(cls, dir_path):
        """新建目录"""
        try:
            os.makedirs(dir_path)
        except OSError:
            pass

    @classmethod
    def copy_file(cls, src, dst):
        """拷贝文件"""
        dst_dir = os.path.dirname(dst)
        cls.make_dir(dst_dir)
        shutil.copy(src, dst)

    def check(self):
        """通过上传和下载检查对象存储配置是否正确"""
        try:
            # 检查bucket是否正确
            assert self.is_exist_bucket(), f'{self.bucket_name}: No Such Bucket'
            # CORS 检查
            assert self._cors_check(), f'{self.bucket_name}: CORS设置错误'

            # 生成一个带有随机字符串的内存文件
            temp_file = create_temp_file(text_length=32)
            text = temp_file.getvalue().decode()

            key = f'storage_check_{text}.txt'
            # 上传
            file_url = self.upload(temp_file, key=key)
            logger.debug(f'upload: {file_url}')
            assert file_url, f'{self.bucket_name}: Upload Failed'
            # 加签url
            assert self._check_sign_url(key), f'{self.bucket_name}: Sign Url Error'
            # 下载
            download_file = self.download(key=f'storage_check_{text}.txt', is_stream=True)
            download_text = download_file.read().decode()
            assert download_text == text, f'{self.bucket_name}: DownloadFailed'

            # 获取文件元数据
            metadata = self.get_object_meta(key)
            logger.debug(f'get_object_meta: {metadata}')
            assert metadata, f'{self.bucket_name}: Get object metadata Failed'
            # 修改文件元数据
            assert self.update_file_headers(key, {'Content-Type': 'text/plain'}), f'{self.bucket_name}: Update object metadata Failed'
            logger.debug(f'update_file_headers: {self.get_object_meta(key)}')

            # 遍历文件
            objects = self.iter_objects(key)
            logger.debug(f'iter_objects: {objects}')
            assert objects, f'{self.bucket_name} iter objects Failed'

            return True
        except AssertionError as e:
            raise StorageRequestError(e)

    def _cors_check(self):
        """检查存储桶的CORS配置是否设置正确"""
        allowed_methods = {'GET', 'PUT', 'POST', 'DELETE', 'HEAD'}
        cors_dict = self.get_bucket_cors()
        logger.debug(f'_cors_check: {cors_dict}')
        if set(cors_dict['allowed_methods']) != allowed_methods:
            raise StorageRequestError(f'{self.bucket_name}: CORS设置错误')
        if cors_dict['allowed_headers'] != ['*']:
            raise StorageRequestError(f'{self.bucket_name}: CORS设置错误')
        if cors_dict['allowed_origins'] != ['*']:
            raise StorageRequestError(f'{self.bucket_name}: CORS设置错误')
        return True

    def _check_sign_url(self, key):
        """判断加签url是否可以正常打开，并且配置了https"""
        try:
            sign_url = self.get_sign_url(key=key, expire=600)
            resp = urlopen('https:' + sign_url)
            assert resp.status < 300, f'{self.bucket_name}: Sign Url Error, {sign_url}'
        except URLError as e:
            if isinstance(e.reason, SSLCertVerificationError):
                raise StorageRequestError(f'{self.bucket_name}: 未开启https')
            raise StorageRequestError(f'{self.bucket_name}: Sign Url Error')
        return True

    @staticmethod
    def parse_content_type(filename):
        ext = filename.split('.')[-1].lower()
        return CONTENT_TYPE.get(ext, 'application/octet-stream')

    def get_key_from_url(self, url):
        """从URL中获取对象存储key"""
        if not url.startswith('//') or not url.startswith('http'):
            url = '//' + url
        url_parse = urlparse(url)
        return url_parse.path[1:]  # 去掉最前面的 /

    @staticmethod
    def get_filename(key):
        """从key中提取文件名，不包含路径"""
        key = unquote(key)
        return key.split('/')[-1]



import functools


def wrap_request_return_bool(func):
    """"""

    @functools.wraps(func)
    def wrap_func(*args, **kwargs):
        try:
            resp = func(*args, **kwargs)
            if resp.status < 300:
                return True
            else:
                return False
        except:
            import traceback
            print(traceback.format_exc())

    return wrap_func



IMAGE_FORMAT_SET = [
    'bmp', 'jpg', 'jpeg', 'png', 'tif', 'gif', 'pcx', 'tga',
    'exif', 'fpx', 'svg', 'psd', 'cdr', 'pcd', 'dxf', 'ufo',
    'eps', 'ai', 'raw', 'WMF', 'webp', 'tiff'
]

CONTENT_TYPE = {
    'jpg': 'image/jpeg',
    'jpeg': 'image/jpeg',
    'jfif': 'image/jpeg',
    'jpe': 'image/jpeg',
    'gif': 'image/gif',
    'png': 'image/png',
    'tif': 'image/tiff',
    'tiff': 'image/tiff',

    'dds': 'application/octet-stream',
}



from yzcore.extensions.storage.oss import OssManager
from yzcore.extensions.storage.obs import ObsManager
from yzcore.extensions.storage.minio import MinioManager
from yzcore.extensions.storage.base import StorageRequestError
from yzcore.extensions.storage.const import IMAGE_FORMAT_SET


__all__ = [
    'IMAGE_FORMAT_SET',
    'StorageManage',
    'StorageRequestError',
]


class StorageManage(object):
    """
    通用的对象存储封装，根据mode选择oss/obs等等
    mode,
    access_key_id,
    access_key_secret,
    bucket_name,
    endpoint=None,
    cname=None,
    cache_path='.',
    private_expire_time=30,  # 上传签名有效时间
    private_expire_time=30,  # 加签URl有效时间

    """

    def __new__(cls, **kwargs):
        if kwargs['mode'].lower() == 'obs':
            storage_manage = ObsManager(**kwargs)
        elif kwargs['mode'].lower() == 'oss':
            storage_manage = OssManager(**kwargs)
        elif kwargs['mode'].lower() == 'minio':
            storage_manage = MinioManager(**kwargs)
        else:
            storage_manage = None
        return storage_manage



#!/usr/bin/python3.6+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-8-5
@desc: 全局异常类定义
"""
from typing import Any

from fastapi import HTTPException


class NotFoundObject(HTTPException):
    def __init__(self, detail: Any = 'Not Found', headers: dict = None):
        super().__init__(status_code=404, detail=detail, headers=headers)


class MultiObjects(HTTPException):
    def __init__(self, detail: Any = '', headers: dict = None):
        detail = '{}存在多个对象，不符合唯一性要求'.format(detail)
        super().__init__(status_code=500, detail=detail, headers=headers)


class CreateObjectFailed(HTTPException):
    def __init__(self, detail: Any = 'Object create failed', headers: dict = None):
        super().__init__(status_code=400, detail=detail, headers=headers)


class UpdateObjectFailed(HTTPException):
    def __init__(self, detail: Any = 'Object update failed', headers: dict = None):
        super().__init__(status_code=400, detail=detail, headers=headers)


class NoObjectCreated(HTTPException):
    def __init__(self, detail: Any = 'No object was created', headers: dict = None):
        super().__init__(status_code=400, detail=detail, headers=headers)


class AlreadyExistObject(HTTPException):
    def __init__(self, detail: Any = 'Already Exist', headers: dict = None):
        super().__init__(status_code=400, detail=detail, headers=headers)


class RequestParamsError(HTTPException):
    def __init__(self, detail: Any = 'Incorrect request parameters', headers: dict = None):
        super().__init__(status_code=400, detail=detail, headers=headers)


class RequestParamsMissing(HTTPException):
    def __init__(self, detail: Any = 'Missing request parameters', headers: dict = None):
        super().__init__(status_code=400, detail=detail, headers=headers)


class NoPermission(HTTPException):
    def __init__(self, detail: Any = 'Insufficient permissions', headers: dict = None):
        super().__init__(status_code=401, detail=detail, headers=headers)


class UnknownError(HTTPException):
    def __init__(self, detail: Any = 'Unknown error', headers: dict = None):
        super().__init__(status_code=500, detail=detail, headers=headers)


if __name__ == '__main__':
    pass



#!/usr/bin/python3.6.8+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-9-27
@desc: 全局常量
"""
from enum import Enum


class ObjectType(Enum):
    ORGANIZ = 1
    PROJECT = 2
    SCENE = 3
    ASSETS = 4


class PermissionType(Enum):
    OWNER = 15
    ADMIN = 7
    EDITOR = 3
    VIEWER = 1



#!/usr/bin/env python3.5+
# -*- coding: utf-8 -*-
"""
@auth: cml
@date: 2019-07-10
@desc: 日志对象的处理函数

内置处理器
logging模块提供了一些处理器，可以通过各种方式处理日志消息。使用addHandler()方法将这些处理器添加给Logger对象。另外还可以为每个处理器配置它自己的筛选和级别。
logging.StreamHandler 可以向类似与sys.stdout或者sys.stderr的任何文件对象(file object)输出信息
logging.FileHandler 将日志消息写入文件filename。
logging.handlers.DatagramHandler(host，port) 发送日志消息给位于制定host和port上的UDP服务器。使用UDP协议，将日志信息发送到网络
logging.handlers.HTTPHandler(host, url) 使用HTTP的GET或POST方法将日志消息上传到一台HTTP 服务器。
logging.handlers.RotatingFileHandler(filename) 将日志消息写入文件filename。如果文件的大小超出maxBytes制定的值，那么它将被备份为filenamel。
logging.handlers.SocketHandler 使用TCP协议，将日志信息发送到网络。
logging.handlers.SysLogHandler 日志输出到syslog
logging.handlers.NTEventLogHandler 远程输出日志到Windows NT/2000/XP的事件日志
logging.handlers.SMTPHandler 远程输出日志到邮件地址
logging.handlers.MemoryHandler 日志输出到内存中的制定buffer
"""
from logging import StreamHandler, FileHandler
from logging.handlers import RotatingFileHandler, TimedRotatingFileHandler, SMTPHandler
import fcntl, time, os, codecs, string, re, types, pickle, struct, shutil
from stat import ST_DEV, ST_INO, ST_MTIME


class StreamHandlerMP(StreamHandler):
    """
    A handler class which writes logging records, appropriately formatted,
    to a stream. Use for multiprocess.
    """

    def emit(self, record):
        """
        Emit a record.
        First seek the end of file for multiprocess to log to the same file
        寻找文件结尾以供多进程登录到同一文件
        """
        try:
            if hasattr(self.stream, "seek"):
                self.stream.seek(0, os.SEEK_END)
        except IOError as e:
            pass

        StreamHandler.emit(self, record)


class FileHandlerMP(FileHandler, StreamHandlerMP):
    """
    A handler class which writes formatted logging records to disk files
        for multiprocess
    """

    def emit(self, record):
        """
        Emit a record.

        If the stream was not opened because 'delay' was specified in the
        constructor, open it before calling the superclass's emit.
        如果由于在构造函数中指定了“delay”而未打开流，请在调用超类的emit之前将其打开。
        """
        if self.stream is None:
            self.stream = self._open()
        StreamHandlerMP.emit(self, record)


class RotatingFileHandlerMP(RotatingFileHandler, FileHandlerMP):
    """
    Handler for logging to a set of files, which switches from one file
    to the next when the current file reaches a certain size.

    Based on logging.RotatingFileHandler, modified for Multiprocess
    """
    _lock_dir = '.lock'
    if os.path.exists(_lock_dir):
        pass
    else:
        os.mkdir(_lock_dir)

    def doRollover(self):
        """
        Do a rollover, as described in __init__().
        For multiprocess, we use shutil.copy instead of rename.
        """

        self.stream.close()
        if self.backupCount > 0:
            for i in range(self.backupCount - 1, 0, -1):
                sfn = "%s.%d" % (self.baseFilename, i)
                dfn = "%s.%d" % (self.baseFilename, i + 1)
                if os.path.exists(sfn):
                    if os.path.exists(dfn):
                        os.remove(dfn)
                    shutil.copy(sfn, dfn)
            dfn = self.baseFilename + ".1"
            if os.path.exists(dfn):
                os.remove(dfn)
            if os.path.exists(self.baseFilename):
                shutil.copy(self.baseFilename, dfn)
        self.mode = 'w'
        self.stream = self._open()

    def emit(self, record):
        """
        Emit a record.

        Output the record to the file, catering for rollover as described
        in doRollover().

        For multiprocess, we use file lock. Any better method ?
        """
        try:
            if self.shouldRollover(record):
                self.doRollover()
            FileLock = self._lock_dir + '/' + os.path.basename(self.baseFilename) + '.' + record.levelname
            f = open(FileLock, "w+")
            fcntl.flock(f.fileno(), fcntl.LOCK_EX)
            FileHandlerMP.emit(self, record)
            fcntl.flock(f.fileno(), fcntl.LOCK_UN)
            f.close()
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)


class TimedRotatingFileHandlerMP(TimedRotatingFileHandler, FileHandlerMP):
    """
    Handler for logging to a file, rotating the log file at certain timed
    intervals.

    If backupCount is > 0, when rollover is done, no more than backupCount
    files are kept - the oldest ones are deleted.
    """
    _lock_dir = '.lock'
    if os.path.exists(_lock_dir):
        pass
    else:
        os.mkdir(_lock_dir)

    def __init__(self, filename, when='h', interval=1, backupCount=0, encoding=None, delay=0, utc=0):
        FileHandlerMP.__init__(self, filename, 'a', encoding, delay)
        self.encoding = encoding
        self.when = when.upper()
        self.backupCount = backupCount
        self.utc = utc
        # Calculate the real rollover interval, which is just the number of
        # seconds between rollovers.  Also set the filename suffix used when
        # a rollover occurs.  Current 'when' events supported:
        # S - Seconds
        # M - Minutes
        # H - Hours
        # D - Days
        # midnight - roll over at midnight
        # W{0-6} - roll over on a certain day; 0 - Monday
        #
        # Case of the 'when' specifier is not important; lower or upper case
        # will work.
        if self.when == 'S':
            self.suffix = "%Y-%m-%d_%H-%M-%S"
            self.extMatch = r"^\d{4}-\d{2}-\d{2}_\d{2}-\d{2}-\d{2}$"
        elif self.when == 'M':
            self.suffix = "%Y-%m-%d_%H-%M"
            self.extMatch = r"^\d{4}-\d{2}-\d{2}_\d{2}-\d{2}$"
        elif self.when == 'H':
            self.suffix = "%Y-%m-%d_%H"
            self.extMatch = r"^\d{4}-\d{2}-\d{2}_\d{2}$"
        elif self.when == 'D' or self.when == 'MIDNIGHT':
            self.suffix = "%Y-%m-%d"
            self.extMatch = r"^\d{4}-\d{2}-\d{2}$"
        elif self.when.startswith('W'):
            if len(self.when) != 2:
                raise ValueError("You must specify a day for weekly rollover from 0 to 6 (0 is Monday): %s" % self.when)
            if self.when[1] < '0' or self.when[1] > '6':
                raise ValueError("Invalid day specified for weekly rollover: %s" % self.when)
            self.dayOfWeek = int(self.when[1])
            self.suffix = "%Y-%m-%d"
            self.extMatch = r"^\d{4}-\d{2}-\d{2}$"
        else:
            raise ValueError("Invalid rollover interval specified: %s" % self.when)

        self.extMatch = re.compile(self.extMatch)

        if interval != 1:
            raise ValueError("Invalid rollover interval, must be 1")

    def shouldRollover(self, record):
        """
        Determine if rollover should occur.

        record is not used, as we are just comparing times, but it is needed so
        the method signatures are the same
        """
        if not os.path.exists(self.baseFilename):
            # print "file don't exist"
            return 0

        cTime = time.localtime(time.time())
        mTime = time.localtime(os.stat(self.baseFilename)[ST_MTIME])
        if self.when == "S" and cTime[5] != mTime[5]:
            # print "cTime:", cTime[5], "mTime:", mTime[5]
            return 1
        elif self.when == 'M' and cTime[4] != mTime[4]:
            # print "cTime:", cTime[4], "mTime:", mTime[4]
            return 1
        elif self.when == 'H' and cTime[3] != mTime[3]:
            # print "cTime:", cTime[3], "mTime:", mTime[3]
            return 1
        elif (self.when == 'MIDNIGHT' or self.when == 'D') and cTime[2] != mTime[2]:
            # print "cTime:", cTime[2], "mTime:", mTime[2]
            return 1
        elif self.when == 'W' and cTime[1] != mTime[1]:
            # print "cTime:", cTime[1], "mTime:", mTime[1]
            return 1
        else:
            return 0

    def doRollover(self):
        """
        do a rollover; in this case, a date/time stamp is appended to the filename
        when the rollover happens.  However, you want the file to be named for the
        start of the interval, not the current time.  If there is a backup count,
        then we have to get a list of matching filenames, sort them and remove
        the one with the oldest suffix.

        For multiprocess, we use shutil.copy instead of rename.
        """
        if self.stream:
            self.stream.close()
        # get the time that this sequence started at and make it a TimeTuple
        # t = self.rolloverAt - self.interval
        t = int(time.time())
        if self.utc:
            timeTuple = time.gmtime(t)
        else:
            timeTuple = time.localtime(t)
        dfn = self.baseFilename + "." + time.strftime(self.suffix, timeTuple)
        if os.path.exists(dfn):
            os.remove(dfn)
        if os.path.exists(self.baseFilename):
            shutil.copy(self.baseFilename, dfn)
            # print "%s -> %s" % (self.baseFilename, dfn)
            # os.rename(self.baseFilename, dfn)
        if self.backupCount > 0:
            # find the oldest log file and delete it
            # s = glob.glob(self.baseFilename + ".20*")
            # if len(s) > self.backupCount:
            #    s.sort()
            #    os.remove(s[0])
            for s in self.getFilesToDelete():
                os.remove(s)
        self.mode = 'w'
        self.stream = self._open()

    def emit(self, record):
        """
        Emit a record.

        Output the record to the file, catering for rollover as described
        in doRollover().

        For multiprocess, we use file lock. Any better method ?
        """
        try:
            if self.shouldRollover(record):
                self.doRollover()
            FileLock = self._lock_dir + '/' + os.path.basename(self.baseFilename) + '.' + record.levelname
            f = open(FileLock, "w+")
            fcntl.flock(f.fileno(), fcntl.LOCK_EX)
            FileHandlerMP.emit(self, record)
            fcntl.flock(f.fileno(), fcntl.LOCK_UN)
            f.close()
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)



#!/usr/bin/python3.6.8+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-12-30
@desc: 时间方面的处理函数
"""
import time
import datetime


__all__ = [
    "get_zero_time",
    "get_today_date",
    "datetime2timestamp",
    "timestamp2datetime",
]

def get_zero_time(
        date_time: datetime = None,
        is_timestamp: bool = True
):
    """
    获取0点时间，默认获取今天的0点时间

    :param date_time:
    :param is_timestamp:
    :return:
    """
    if not date_time:
        date_time = datetime.date.today()
        if is_timestamp:
            return int(time.mktime(date_time.timetuple()))
        return date_time
    else:
        time_stamp = int(time.mktime(date_time.timetuple()))
        if is_timestamp:
            return time_stamp
        date_time = datetime.datetime.fromtimestamp(time_stamp)
    return date_time


def datetime2timestamp(date_time: datetime):
    return int(time.mktime(date_time.timetuple()))


def timestamp2datetime(time_stamp: int):
    _s = str(time_stamp)
    if len(_s) > 10 and '.' not in _s:
        _s = '.'.join((_s[:10], _s[10:]))
        time_stamp = float(_s)
    return datetime.datetime.fromtimestamp(time_stamp)


def get_today_date(is_strf: bool = False, format: str = "%Y-%m-%d"):
    """
    获取今天的日期

    :param is_strf:
    :param format:
    :return:
    """
    today = datetime.date.today()
    if is_strf:
        today = today.strftime(format)
    return today




#!/usr/bin/env python3.5+
# -*- coding: utf-8 -*-
"""
@auth: cml
@date: 2019-07-10
@desc: ...

日志的配置参数，日志对象主要有3个子模块，分别为
    - formaters（输出格式）
    - handlers（日志操作类型）
    - loggers（日志记录器的名称）

其中hadlers为日志的具体执行，依赖于formaters和日志操作类型或一些属性，
比如按大小分片还是时间分片，是写入还是打印到控制台。
logger负责调用handle，一个logger可以调用多个handler，
比如logger.info调用了打印到控制台handler（logging.StreamHandler）
和写入到文件handler（mlogging.TimedRotatingFileHandler_MP），
在没有指定logger名字的时候，即logger=logging.get_logger()的时候，
logger会自动选择名为root的logger

format: 日志消息格式
格式                 描述
%(name)s            记录器的名称
%(levelno)s         数字形式的日志记录级别
%(levelname)s       日志记录级别的文本名称
%(filename)s        执行日志记录调用的源文件的文件名称
%(pathname)s        执行日志记录调用的源文件的路径名称
%(funcName)s        执行日志记录调用的函数名称
%(module)s          执行日志记录调用的模块名称
%(lineno)s          执行日志记录调用的行号
%(created)s         执行日志记录的时间
%(asctime)s         日期和时间
%(msecs)s           毫秒部分
%(thread)d          线程ID
%(threadName)s      线程名称
%(process)d         进程ID
%(message)s         记录的消息


LogRecord 属性:

| 属性名称        | 格式                  | 描述                                                         |
| :-------------- | :-------------------- | :----------------------------------------------------------- |
| args            | 不需要格式化。          | 合并到 `msg` 以产生 `message` 的包含参数的元组，或是其中的值将被用于合并的字典（当只有一个参数且其类型为字典时）。 |
| asctime         | `%(asctime)s`         | 表示 [`LogRecord`](https://docs.python.org/zh-cn/3/library/logging.html#logging.LogRecord) 何时被创建的供人查看时间值。 默认形式为 '2003-07-08 16:49:45,896' (逗号之后的数字为时间的毫秒部分)。 |
| created         | `%(created)f`         | [`LogRecord`](https://docs.python.org/zh-cn/3/library/logging.html#logging.LogRecord) 被创建的时间（即 [`time.time()`](https://docs.python.org/zh-cn/3/library/time.html#time.time) 的返回值）。 |
| exc_info        | 不需要格式化。          | 异常元组 (例如 `sys.exc_info`) 或者如未发生异常则为 `None`。 |
| 文件名           | `%(filename)s`        | `pathname` 的文件名部分。                                    |
| funcName        | `%(funcName)s`        | 函数名包括调用日志记录.                                      |
| levelname       | `%(levelname)s`       | 消息文本记录级别 (`'DEBUG'`, `'INFO'`, `'WARNING'`, `'ERROR'`, `'CRITICAL'`). |
| levelno         | `%(levelno)s`         | 消息数字记录级别 (`DEBUG`, `INFO`, `WARNING`, `ERROR`, `CRITICAL`). |
| lineno          | `%(lineno)d`          | 发出日志记录调用所在的源行号（如果可用）。                   |
| message         | `%(message)s`         | 记入日志的消息，即 `msg % args` 的结果。 这是在发起调用 [`Formatter.format()`](https://docs.python.org/zh-cn/3/library/logging.html#logging.Formatter.format) 时设置的。 |
| module          | `%(module)s`          | 模块 (`filename` 的名称部分)。                               |
| msecs           | `%(msecs)d`           | [`LogRecord`](https://docs.python.org/zh-cn/3/library/logging.html#logging.LogRecord) 被创建的时间的毫秒部分。 |
| msg             | 不需要格式化。          | 在原始日志记录调用中传入的格式字符串。 与 `args` 合并以产生 `message`，或是一个任意对象 (参见 [使用任意对象作为消息](https://docs.python.org/zh-cn/3/howto/logging.html#arbitrary-object-messages))。 |
| 名称             | `%(name)s`            | 用于记录调用的日志记录器名称。                               |
| pathname        | `%(pathname)s`        | 发出日志记录调用的源文件的完整路径名（如果可用）。           |
| process         | `%(process)d`         | 进程ID（如果可用）                                           |
| processName     | `%(processName)s`     | 进程名（如果可用）                                           |
| relativeCreated | `%(relativeCreated)d` | 以毫秒数表示的 LogRecord 被创建的时间，即相对于 logging 模块被加载时间的差值。 |
| stack_info      | 不需要格式化。          | 当前线程中从堆栈底部起向上直到包括日志记录调用并导致创建此记录的堆栈帧的堆栈帧信息（如果可用）。 |
| thread          | `%(thread)d`          | 线程ID（如果可用）                                           |
| threadName      | `%(threadName)s`      | 线程名（如果可用）                                           |
"""
import os
import sys
curr_path = os.path.abspath(os.path.dirname(os.curdir))
sys.path.append(curr_path)
# _path = os.path.join(os.path.dirname(os.path.dirname(curr_path)), 'output')
LOG_PATH = os.path.join(curr_path, 'logs')
# from src.settings import log_conf
# LOG_PATH = log_conf.get('log_path')

from .filters import *

LOGGING_CONFIG = {
    "version": 1,
    "disable_existing_loggers": False,  # 不禁用完成配置之前创建的所有日志处理器
    "formatters": {
        "simple": {
            # 简单的输出模式
            'format': '%(asctime)s | %(levelname)s | PID:%(process)d | TID:%(threadName)s | [%(module)s: %(funcName)s] | - %(message)s',
            'datefmt': '%Y-%m-%d %H:%M:%S'
        },
        'standard': {
            # 较为复杂的输出模式，可以进行自定义
            'format': '%(asctime)s | %(levelname)s | PID:%(process)d | TID:%(threadName)s | [%(module)s: %(funcName)s] | - %(message)s'
        },
        'custom': {
            # 自定义json输出
            'format': '{"level": "%(levelname)s", "now": "%(created)f", "PID": "%(process)d", "pathname": "%(pathname)s", "lineno": "%(lineno)d", "message": "%(message)s"}'
        },
    },

    # 过滤器
    "filters": {
        'debug_filter': {
            '()': DebugFilter
        },
        'info_filter': {
            '()': InfoFilter
        },
        'warning_filter': {
            '()': WarningFilter
        },
        'error_filter': {
            '()': ErrorFilter
        },
        'critical_filter': {
            '()': CriticalFilter
        },
        'notset_filter': {
            '()': NotSetFilter
        }
    },
    "handlers": {
        # # 输出到控制台的handler
        # "console": {
        #     # 定义输出流的类
        #     "class": "logging.StreamHandler",
        #     # handler等级，如果实际执行等级高于此等级，则不触发handler
        #     "level": "DEBUG",
        #     # 输出的日志格式
        #     "formatter": "simple",
        #     # 流调用系统输出
        #     "stream": "ext://sys.stdout"
        # },
        # # 写入到文件的hanler，写入等级为info，命名为request是为了专门记录一些网络请求日志
        # "file": {
        #     # 定义写入文件的日志类，此类为按时间分割日志类，还有一些按日志大小分割日志的类等
        #     "class": "handlers.TimedRotatingFileHandlerMP",
        #     # 日志等级
        #     "level": "DEBUG",
        #     # 日志写入格式，因为要写入到文件后期可能会debug用，所以用了较为详细的standard日志格式
        #     "formatter": "standard",
        #     # 要写入的文件名
        #     "filename": os.path.join(LOG_PATH, 'default', 'info.log'),
        #     # 分割单位，S秒，D日，H小时，M分钟，W0-W6星期，星期一为 'W0'
        #     # 比如文件名为test.log，到凌晨0点的时候会自动分离出test.log.yyyy-mm-dd
        #     "when": 'D',
        #     "interval": 1,
        #     'backupCount': 5,  # 备份份数
        #     "encoding": "utf8",
        #     "filters": ["info_filter"]
        # },
        # "info_file": {
        #     "class": "handlers.TimedRotatingFileHandlerMP",
        #     "level": "INFO",
        #     "formatter": "standard",
        #     "filename": os.path.join(LOG_PATH, 'default', 'info.log'),
        #     "when": 'D',
        #     'backupCount': 5,  # 备份份数
        #     "encoding": "utf8",
        #     "filters": ["info_filter"]
        # },
        # "err_file": {
        #     "class": "handlers.TimedRotatingFileHandlerMP",
        #     "level": "WARN",
        #     "formatter": "standard",
        #     "filename": os.path.join(LOG_PATH, 'default', 'error.log'),
        #     "when": 'D',
        #     'backupCount': 5,  # 备份份数
        #     "encoding": "utf8",
        #     "filters": ["info_filter"]
        # },
    },
    # "loggers": {},
    "loggers": {
        # # logger名字
        # "default_logger": {
        #     # logger集成的handler
        #     'handlers': ['console', 'file'],
        #     # logger等级，如果实际执行等级，高于此等级，则不触发此logger，logger中所有的handler均不会被触发
        #     'level': "DEBUG",
        #     # 是否继承root日志，如果继承，root的handler会加入到当前logger的handlers中
        #     'propagate': False
        # },
        # "debug_logger": {
        #     'handlers': ['console', 'debug_file'],
        #     'level': "DEBUG",
        #     'propagate': False
        # },
        # "info_logger": {
        #     'handlers': ['console', 'info_file'],
        #     'level': "INFO",
        #     'propagate': False
        # },
        # "warn_logger": {
        #     'handlers': ['console', 'err_file'],
        #     'level': "WARN",
        #     'propagate': False
        # },
        # "error_logger": {
        #     'handlers': ['console', 'err_file'],
        #     'level': "ERROR",
        #     'propagate': False
        # },
        # "critical_logger": {
        #     'handlers': ['console', 'err_file'],
        #     'level': "DEBUG",
        #     'propagate': False
        # },
    },
    # 基础logger，当不指定logger名称时，默认选择此logger
    # "root": {
    #     'handlers': ['file'],
    #     'level': "DEBUG",
    #     'propagate': True
    # },
    # 增量添加配置，默认不增量。
    # "incremental": False
}



#!/usr/bin/python3.7+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-9-
@desc: ...
"""
import os


class CommandError(Exception):
    pass


def execute_from_command_line(argv=None):
    """Run a ManagementUtility."""
    # utility = ManagementUtility(argv)
    # utility.execute()
    from yzrpc.commands import CommandUtility
    if argv[0] == 'yzrpc':
        utility = CommandUtility()
    else:
        utility = CommandUtility(
            command_dir=os.path.join(os.path.dirname(__file__), "commands"),
            module_path="yzcore.core.management.commands"
        )
    utility.run_from_argv(argv)
